// Unified token -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "Token.h"
#include "VariableSet.h"

/** @file Token.C
 * Tokens used in the unification process
 */

Token::~Token ()
{
  delete myUnifiedUndefined;
}

bool
Token::isBindable (const class Valuation& valuation)
{
  assert (valuation.isOK ());
  if (class Value* value = myUnifier.m->getToken ()->eval (valuation)) {
    assert (valuation.isOK ());
    myIterator = myPlaceMarking.find (value);
    delete value;
    if (myIterator != end ()) {
      if (myCardinality <= PlaceMarking::getCount (myIterator))
	return true;
      myIterator = begin ();
    }
  }

  return false;
}

bool
Token::isBindableUnfold (const class Valuation& valuation)
{
  assert (valuation.isOK ());
  if (class Value* value = myUnifier.m->getToken ()->eval (valuation)) {
    assert (valuation.isOK ());
    myIterator = myPlaceMarking.find (value);
    delete value;
    return myIterator != end ();
  }

  return false;
}

void
Token::addUnified (const class VariableDefinition& var)
{
  if (!myUnifiedUndefined) myUnifiedUndefined = new class VariableSet;
  myUnifiedUndefined->insert (var);
}

bool
Token::isUnified (const class VariableDefinition& var) const
{
  return myUnifiedUndefined && myUnifiedUndefined->contains (var);
}
