// Parallelized, lossless reachability set storage -*- c++ -*-

#ifndef PARSET_H_
# define PARSET_H_
# ifdef __GNUC__
#  pragma interface
# endif // __GNUC__

# include "StateSet.h"
# include "ByteBuffer.h"
# include "file.h"

/** @file ParSet.h
 * Transient, parallelized lossless reachability set storage
 */

/** Parallelized reachability set storage */
class ParSet : public StateSet
{
public:
  /** Constructor
   * @param compress	flag: apply path compression
   * @param s		the communications socket
   */
  ParSet (bool compress,
	  int s);
private:
  /** Copy constructor */
  ParSet (const class ParSet& old);
  /** Assignment operator */
  class ParSet& operator= (const class ParSet& old);
public:
  /** Destructor */
  ~ParSet ();

  /** Display path to a rejected state
   * @param dstate	the encoded rejected state (0=deadlock)
   * @param dlen	length of dstate in bytes
   * @param prop	a safety property to be checked (optional)
   * @param net		the net
   * @param printer	the output stream for counterexample
   * @param visual	visualization level
   * @param reason	reason for rejecting the state
   */
  void reject (const void* dstate,
	       size_t dlen,
	       const class Property* prop,
	       const class Net& net,
	       const class Printer& printer,
	       card_t visual,
	       enum StateSet::Code reason) const;
 
protected:
  /** Add a state to the graph
   * @param state	the encoded state
   * @param size	length of the encoded state, in bytes
   * @return		true if the state was enqueued to mySearch
   */
  bool do_add (const void* state,
	       size_t size);

public:
  /** Fetch an encoded state
   * @param tail	flag: retrieve from tail of list instead of head
   * @param size	(output) length of the encoded stream (optional)
   * @return		the encoded state
   */
  word_t* pop (bool tail, size_t* size);

private:
  /** The socket file descriptor */
  int myFD;
  /** The reception buffer */
  mutable class BytePacker myRecv;
  /** The transmission buffer */
  mutable class BytePacker mySend;
  /** Index to the first unread byte in myRecv */
  mutable unsigned myRecvCount;
  /** Index to the first unsent byte in mySend */
  mutable unsigned mySendCount;
};

#endif // PARSET_H_
