// Parallelized safety property analysis -*- c++ -*-

/**
 * @file server.h
 * Parallelized safety property analysis
 */

#ifndef SERVER_H_
# define SERVER_H_

/** fork client processes
 * @param num		number of clients to fork
 * @param states	(input/output) the set of generated states
 * @param prop		safety property being checked (optional)
 * @param net		net being checked
 * @param printer	printer object for diagnostics
 * @param visual	visualization level
 * @param breadth	flag: apply breadth-first search
 * @param compress	flag: apply path compression
 * @return		true for the server; false for others
 */
bool
createJobs (unsigned num,
	    class StateSet*& states,
	    const class Property* prop,
	    const class Net& net,
	    const class Printer& printer,
	    card_t visual,
	    bool breadth,
	    bool compress);

/** Resolve a TCP/IP port/address string.
 * @param address	the address to be resolved
 * @return		pointer to the corresponding structure, or 0 on error
 */
struct sockaddr_in*
resolve (const char* address);

/** Connect to the server process specified by resolve ().
 * @param states	(output) the set of generated states
 * @param compress	flag: apply path compression
 * @return		0 for clients; 1 for the server; other values on error
 */
int
client_connect (class StateSet*& states,
		bool compress);

/** Clean up the communications structures */
void
comm_cleanup (void);

#endif // SERVER_H_
